@@ -14,23 +14,21 @@
 
 import logging
 
-from copy import deepcopy
 from numba import njit
 
 import numpy as np
 
-from aesara import function as aesara_function
-from aesara import config
-from aesara.tensor.var import Variable
+from pytensor import function as pytensor_function
+from pytensor import config
+from pytensor.tensor.var import Variable
 
 from pymc.model import modelcontext
-from pymc.step_methods.arraystep import ArrayStepShared, Competence
-from pymc.aesaraf import inputvars, join_nonshared_inputs, make_shared_replacements
-
+from pymc.step_methods.arraystep import ArrayStepShared
+from pymc.step_methods.compound import Competence
+from pymc.pytensorf import inputvars, join_nonshared_inputs, make_shared_replacements
 
 from pymc_bart.bart import BARTRV
-from pymc_bart.tree import LeafNode, SplitNode, Tree
-
+from pymc_bart.tree import Tree, Node, get_depth
 
 _log = logging.getLogger("pymc")
 
@@ -108,7 +106,7 @@ class PGBART(ArrayStepShared):
             config.floatX
         )
         self.sum_trees_noi = self.sum_trees - (init_mean / self.m)
-        self.a_tree = Tree(
+        self.a_tree = Tree.new_tree(
             leaf_node_value=init_mean / self.m,
             idx_data_points=np.arange(self.num_observations, dtype="int32"),
             num_observations=self.num_observations,
@@ -137,11 +135,7 @@ class PGBART(ArrayStepShared):
 
         shared = make_shared_replacements(initial_values, vars, model)
         self.likelihood_logp = logp(initial_values, [model.datalogp], vars, shared)
-        self.all_particles = []
-        for _ in range(self.m):
-            self.a_tree.leaf_node_value = init_mean / self.m
-            p = ParticleTree(self.a_tree)
-            self.all_particles.append(p)
+        self.all_particles = list(ParticleTree(self.a_tree) for _ in range(self.m))
         self.all_trees = np.array([p.tree for p in self.all_particles])
         super().__init__(vars, shared)
 
@@ -193,18 +187,18 @@ class PGBART(ArrayStepShared):
 
             _, normalized_weights = self.normalize(particles)
             # Get the new tree and update
-            new_particle, new_tree = self.get_particle_tree(particles, normalized_weights)
-            self.all_particles[tree_id] = new_particle
+            self.all_particles[tree_id], new_tree = self.get_particle_tree(
+                particles, normalized_weights
+            )
             self.sum_trees = self.sum_trees_noi + new_tree._predict()
             self.all_trees[tree_id] = new_tree.trim()
-            used_variates = new_tree.get_split_variables()
 
             if self.tune:
                 self.ssv = SampleSplittingVariable(self.alpha_vec)
-                for index in used_variates:
+                for index in new_tree.get_split_variables():
                     self.alpha_vec[index] += 1
             else:
-                for index in used_variates:
+                for index in new_tree.get_split_variables():
                     variable_inclusion[index] += 1
 
         if not self.tune:
@@ -242,7 +236,7 @@ class PGBART(ArrayStepShared):
         new_particles = []
         for idx in new_indices:
             if idx in seen:
-                new_particles.append(deepcopy(particles[idx]))
+                new_particles.append(particles[idx].copy())
             else:
                 new_particles.append(particles[idx])
                 seen.append(idx)
@@ -277,7 +271,7 @@ class PGBART(ArrayStepShared):
     def init_particles(self, tree_id: int) -> np.ndarray:
         """Initialize particles."""
         p0 = self.all_particles[tree_id]
-        p1 = deepcopy(p0)
+        p1 = p0.copy()
         p1.sample_leafs(
             self.sum_trees,
             self.m,
@@ -290,12 +284,9 @@ class PGBART(ArrayStepShared):
         particles = [p0, p1]
 
         for _ in self.indices:
-            pt = ParticleTree(self.a_tree)
-            if self.tune:
-                pt.kfactor = self.uniform.random()
-            else:
-                pt.kfactor = p0.kfactor
-            particles.append(pt)
+            particles.append(
+                ParticleTree(self.a_tree, self.uniform.random() if self.tune else p0.kfactor)
+            )
 
         return np.array(particles)
 
@@ -311,10 +302,10 @@ class PGBART(ArrayStepShared):
         )
         if old:
             particle.log_weight = new_likelihood
-            particle.old_likelihood_logp = new_likelihood
         else:
             particle.log_weight += new_likelihood - particle.old_likelihood_logp
-            particle.old_likelihood_logp = new_likelihood
+
+        particle.old_likelihood_logp = new_likelihood
 
     @staticmethod
     def competence(var, has_grad):
@@ -330,12 +321,20 @@ class ParticleTree:
 
     __slots__ = "tree", "expansion_nodes", "log_weight", "old_likelihood_logp", "kfactor"
 
-    def __init__(self, tree):
-        self.tree = tree.copy()  # keeps the tree that we care at the moment
+    def __init__(self, tree, kfactor=0.75):
+        self.tree = tree.copy()
         self.expansion_nodes = [0]
         self.log_weight = 0
         self.old_likelihood_logp = 0
-        self.kfactor = 0.75
+        self.kfactor = kfactor
+
+    def copy(self):
+        p = ParticleTree(self.tree)
+        p.expansion_nodes = self.expansion_nodes.copy()
+        p.log_weight = self.log_weight
+        p.old_likelihood_logp = self.old_likelihood_logp
+        p.kfactor = self.kfactor
+        return p
 
     def sample_tree(
         self,
@@ -353,10 +352,10 @@ class ParticleTree:
         if self.expansion_nodes:
             index_leaf_node = self.expansion_nodes.pop(0)
             # Probability that this node will remain a leaf node
-            prob_leaf = prior_prob_leaf_node[self.tree[index_leaf_node].depth]
+            prob_leaf = prior_prob_leaf_node[get_depth(index_leaf_node)]
 
             if prob_leaf < np.random.random():
-                index_selected_predictor = grow_tree(
+                idx_new_nodes = grow_tree(
                     self.tree,
                     index_leaf_node,
                     ssv,
@@ -369,9 +368,8 @@ class ParticleTree:
                     self.kfactor,
                     shape,
                 )
-                if index_selected_predictor is not None:
-                    new_indexes = self.tree.idx_leaf_nodes[-2:]
-                    self.expansion_nodes.extend(new_indexes)
+                if idx_new_nodes is not None:
+                    self.expansion_nodes.extend(idx_new_nodes)
                     tree_grew = True
 
         return tree_grew
@@ -385,8 +383,7 @@ class ParticleTree:
                 node_value = draw_leaf_value(
                     sum_trees[:, idx_data_points],
                     m,
-                    normal,
-                    self.kfactor,
+                    normal.random() * self.kfactor,
                     shape,
                 )
                 leaf.value = node_value
@@ -412,7 +409,7 @@ class SampleSplittingVariable:
 
 def compute_prior_probability(alpha):
     """
-    Calculate the probability of the node being a LeafNode (1 - p(being SplitNode)).
+    Calculate the probability of the node being a leaf node (1 - p(being split node)).
 
     Taken from equation 19 in [Rockova2018].
 
@@ -459,60 +456,46 @@ def grow_tree(
     split_value = get_split_value(available_splitting_values, idx_data_points, missing_data)
 
     if split_value is None:
-        index_selected_predictor = None
-    else:
-        new_idx_data_points = get_new_idx_data_points(
-            split_value, idx_data_points, selected_predictor, X
-        )
-        current_node_children = (
-            current_node.get_idx_left_child(),
-            current_node.get_idx_right_child(),
+        return None
+    new_idx_data_points = get_new_idx_data_points(
+        available_splitting_values, split_value, idx_data_points
+    )
+    current_node_children = (
+        current_node.get_idx_left_child(),
+        current_node.get_idx_right_child(),
+    )
+
+    new_nodes = []
+    for idx in range(2):
+        idx_data_point = new_idx_data_points[idx]
+        node_value = draw_leaf_value(
+            sum_trees[:, idx_data_point],
+            m,
+            normal.random() * kfactor,
+            shape,
         )
 
-        new_nodes = []
-        for idx in range(2):
-            idx_data_point = new_idx_data_points[idx]
-            node_value = draw_leaf_value(
-                sum_trees[:, idx_data_point],
-                m,
-                normal,
-                kfactor,
-                shape,
-            )
-
-            new_node = LeafNode(
-                index=current_node_children[idx],
-                value=node_value,
-                idx_data_points=idx_data_point,
-            )
-            new_nodes.append(new_node)
-
-        new_split_node = SplitNode(
-            index=index_leaf_node,
-            idx_split_variable=selected_predictor,
-            split_value=split_value,
+        new_node = Node.new_leaf_node(
+            index=current_node_children[idx],
+            value=node_value,
+            idx_data_points=idx_data_point,
         )
+        new_nodes.append(new_node)
 
-        # update tree nodes and indexes
-        tree.delete_leaf_node(index_leaf_node)
-        tree.set_node(index_leaf_node, new_split_node)
-        tree.set_node(new_nodes[0].index, new_nodes[0])
-        tree.set_node(new_nodes[1].index, new_nodes[1])
+    tree.grow_leaf_node(current_node, selected_predictor, split_value, index_leaf_node)
+    tree.set_node(new_nodes[0].index, new_nodes[0])
+    tree.set_node(new_nodes[1].index, new_nodes[1])
 
-    return index_selected_predictor
+    return [new_nodes[0].index, new_nodes[1].index]
 
 
-def get_new_idx_data_points(split_value, idx_data_points, selected_predictor, X):
-
-    left_idx = X[idx_data_points, selected_predictor] <= split_value
-    left_node_idx_data_points = idx_data_points[left_idx]
-    right_node_idx_data_points = idx_data_points[~left_idx]
-
-    return left_node_idx_data_points, right_node_idx_data_points
+@njit
+def get_new_idx_data_points(available_splitting_values, split_value, idx_data_points):
+    split_idx = available_splitting_values <= split_value
+    return idx_data_points[split_idx], idx_data_points[~split_idx]
 
 
 def get_split_value(available_splitting_values, idx_data_points, missing_data):
-
     if missing_data:
         idx_data_points = idx_data_points[~np.isnan(available_splitting_values)]
         available_splitting_values = available_splitting_values[
@@ -527,19 +510,18 @@ def get_split_value(available_splitting_values, idx_data_points, missing_data):
     return split_value
 
 
-def draw_leaf_value(y_mu_pred, m, normal, kfactor, shape):
+@njit
+def draw_leaf_value(y_mu_pred, m, norm, shape):
     """Draw Gaussian distributed leaf values."""
     if y_mu_pred.size == 0:
         return np.zeros(shape)
-    else:
-        norm = normal.random() * kfactor
-        if y_mu_pred.size == 1:
-            mu_mean = np.full(shape, y_mu_pred.item() / m)
-        else:
-            mu_mean = fast_mean(y_mu_pred) / m
 
-        draw = norm + mu_mean
-        return draw
+    if y_mu_pred.size == 1:
+        mu_mean = np.full(shape, y_mu_pred.item() / m)
+    else:
+        mu_mean = fast_mean(y_mu_pred) / m
+
+    return norm + mu_mean
 
 
 @njit
@@ -656,9 +638,9 @@ def logp(point, out_vars, vars, shared):  # pylint: disable=redefined-builtin
     vars: List
         containing :class:`pymc.Distribution` for the input variables
     shared: List
-        containing :class:`aesara.tensor.Tensor` for depended shared data
+        containing :class:`pytensor.tensor.Tensor` for depended shared data
     """
     out_list, inarray0 = join_nonshared_inputs(point, out_vars, vars, shared)
-    function = aesara_function([inarray0], out_list[0])
+    function = pytensor_function([inarray0], out_list[0])
     function.trust_input = True
     return function
